@@ -62,18 +62,9 @@ static const char *vpnc_binary_paths[] =
 #define NM_VPNC_UDP_ENCAPSULATION_PORT	0 /* random port */
 #define NM_VPNC_LOCAL_PORT_ISAKMP	0 /* random port */
 
-typedef enum {
-	ITEM_TYPE_UNKNOWN = 0,
-	ITEM_TYPE_IGNORED,
-	ITEM_TYPE_STRING,
-	ITEM_TYPE_BOOLEAN,
-	ITEM_TYPE_INT,
-	ITEM_TYPE_PATH
-} ItemType;
-
 typedef struct {
 	const char *name;
-	guint32 type;
+	GType type;
 	gint int_min;
 	gint int_max;
 } ValidProperty;
@@ -81,35 +72,34 @@ typedef struct {
 #define LEGACY_NAT_KEEPALIVE "NAT-Keepalive packet interval"
 
 static ValidProperty valid_properties[] = {
-	{ NM_VPNC_KEY_GATEWAY,               ITEM_TYPE_STRING, 0, 0 },
-	{ NM_VPNC_KEY_ID,                    ITEM_TYPE_STRING, 0, 0 },
-	{ NM_VPNC_KEY_XAUTH_USER,            ITEM_TYPE_STRING, 0, 0 },
-	{ NM_VPNC_KEY_DOMAIN,                ITEM_TYPE_STRING, 0, 0 },
-	{ NM_VPNC_KEY_DHGROUP,               ITEM_TYPE_STRING, 0, 0 },
-	{ NM_VPNC_KEY_PERFECT_FORWARD,       ITEM_TYPE_STRING, 0, 0 },
-	{ NM_VPNC_KEY_VENDOR,                ITEM_TYPE_STRING, 0, 0 },
-	{ NM_VPNC_KEY_APP_VERSION,           ITEM_TYPE_STRING, 0, 0 },
-	{ NM_VPNC_KEY_SINGLE_DES,            ITEM_TYPE_BOOLEAN, 0, 0 },
-	{ NM_VPNC_KEY_NO_ENCRYPTION,         ITEM_TYPE_BOOLEAN, 0, 0 },
-	{ NM_VPNC_KEY_DPD_IDLE_TIMEOUT,      ITEM_TYPE_INT, 0, 86400 },
-	{ NM_VPNC_KEY_NAT_TRAVERSAL_MODE,    ITEM_TYPE_STRING, 0, 0 },
-	{ NM_VPNC_KEY_CISCO_UDP_ENCAPS_PORT, ITEM_TYPE_INT, 0, 65535 },
-	{ NM_VPNC_KEY_LOCAL_PORT,            ITEM_TYPE_INT, 0, 65535 },
-	/* Hybrid Auth */
-	{ NM_VPNC_KEY_AUTHMODE,              ITEM_TYPE_STRING, 0, 0 },
-	{ NM_VPNC_KEY_CA_FILE,               ITEM_TYPE_PATH, 0, 0 },
+	{ NM_VPNC_KEY_GATEWAY,               G_TYPE_STRING, 0, 0 },
+	{ NM_VPNC_KEY_ID,                    G_TYPE_STRING, 0, 0 },
+	{ NM_VPNC_KEY_XAUTH_USER,            G_TYPE_STRING, 0, 0 },
+	{ NM_VPNC_KEY_DOMAIN,                G_TYPE_STRING, 0, 0 },
+	{ NM_VPNC_KEY_DHGROUP,               G_TYPE_STRING, 0, 0 },
+	{ NM_VPNC_KEY_PERFECT_FORWARD,       G_TYPE_STRING, 0, 0 },
+	{ NM_VPNC_KEY_VENDOR,                G_TYPE_STRING, 0, 0 },
+	{ NM_VPNC_KEY_APP_VERSION,           G_TYPE_STRING, 0, 0 },
+	{ NM_VPNC_KEY_SINGLE_DES,            G_TYPE_BOOLEAN, 0, 0 },
+	{ NM_VPNC_KEY_NO_ENCRYPTION,         G_TYPE_BOOLEAN, 0, 0 },
+	{ NM_VPNC_KEY_DPD_IDLE_TIMEOUT,      G_TYPE_INT, 0, 86400 },
+	{ NM_VPNC_KEY_NAT_TRAVERSAL_MODE,    G_TYPE_STRING, 0, 0 },
+	{ NM_VPNC_KEY_CISCO_UDP_ENCAPS_PORT, G_TYPE_INT, 0, 65535 },
+	{ NM_VPNC_KEY_LOCAL_PORT,            G_TYPE_INT, 0, 65535 },
 	/* Ignored option for internal use */
-	{ NM_VPNC_KEY_SECRET_TYPE,           ITEM_TYPE_IGNORED, 0, 0 },
-	{ NM_VPNC_KEY_XAUTH_PASSWORD_TYPE,   ITEM_TYPE_IGNORED, 0, 0 },
+	{ NM_VPNC_KEY_SECRET_TYPE,           G_TYPE_NONE, 0, 0 },
+	{ NM_VPNC_KEY_XAUTH_PASSWORD_TYPE,   G_TYPE_NONE, 0, 0 },
+	{ NM_VPNC_KEY_SECRET"-flags",        G_TYPE_NONE, 0, 0 },
+	{ NM_VPNC_KEY_XAUTH_PASSWORD"-flags",G_TYPE_NONE, 0, 0 },
 	/* Legacy options that are ignored */
-	{ LEGACY_NAT_KEEPALIVE,              ITEM_TYPE_STRING, 0, 0 },
-	{ NULL,                              ITEM_TYPE_UNKNOWN, 0, 0 }
+	{ LEGACY_NAT_KEEPALIVE,              G_TYPE_STRING, 0, 0 },
+	{ NULL,                              G_TYPE_NONE, 0, 0 }
 };
 
 static ValidProperty valid_secrets[] = {
-	{ NM_VPNC_KEY_SECRET,                ITEM_TYPE_STRING, 0, 0 },
-	{ NM_VPNC_KEY_XAUTH_PASSWORD,        ITEM_TYPE_STRING, 0, 0 },
-	{ NULL,                              ITEM_TYPE_UNKNOWN, 0, 0 }
+	{ NM_VPNC_KEY_SECRET,                G_TYPE_STRING, 0, 0 },
+	{ NM_VPNC_KEY_XAUTH_PASSWORD,        G_TYPE_STRING, 0, 0 },
+	{ NULL,                              G_TYPE_NONE, 0, 0 }
 };
 
 typedef struct ValidateInfo {
@@ -122,8 +112,6 @@ static void
 validate_one_property (const char *key, const char *value, gpointer user_data)
 {
 	ValidateInfo *info = (ValidateInfo *) user_data;
-	ValidProperty *prop = NULL;
-	long int tmp;
 	int i;
 
 	if (*(info->error))
@@ -136,68 +124,56 @@ validate_one_property (const char *key, const char *value, gpointer user_data)
 		return;
 
 	for (i = 0; info->table[i].name; i++) {
-		prop = &info->table[i];
-		if (g_strcmp0 (prop->name, key) == 0)
+		ValidProperty prop = info->table[i];
+		long int tmp;
+
+		if (strcmp (prop.name, key))
+			continue;
+
+		switch (prop.type) {
+		case G_TYPE_NONE:
+			return; /* technically valid, but unused */
+		case G_TYPE_STRING:
+			return; /* valid */
+		case G_TYPE_INT:
+			errno = 0;
+			tmp = strtol (value, NULL, 10);
+			if (errno == 0 && tmp >= prop.int_min && tmp <= prop.int_max)
+				return; /* valid */
+
+			g_set_error (info->error,
+			             NM_VPN_PLUGIN_ERROR,
+			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+			             "invalid integer property '%s' or out of range [%d -> %d]",
+			             key, prop.int_min, prop.int_max);
 			break;
+		case G_TYPE_BOOLEAN:
+			if (!strcmp (value, "yes") || !strcmp (value, "no"))
+				return; /* valid */
+
+			g_set_error (info->error,
+			             NM_VPN_PLUGIN_ERROR,
+			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+			             "invalid boolean property '%s' (not yes or no)",
+			             key);
+			break;
+		default:
+			g_set_error (info->error,
+			             NM_VPN_PLUGIN_ERROR,
+			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+			             "unhandled property '%s' type %s",
+			             key, g_type_name (prop.type));
+			break;
+		}
 	}
 
 	/* Did not find the property from valid_properties or the type did not match */
-	if (!prop || !prop->name) {
+	if (!info->table[i].name) {
 		g_set_error (info->error,
 		             NM_VPN_PLUGIN_ERROR,
 		             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
 		             "property '%s' invalid or not supported",
 		             key);
-		return;
-	}
-
-	/* Validate the property */
-	switch (prop->type) {
-	case ITEM_TYPE_IGNORED:
-		break; /* technically valid, but unused */
-	case ITEM_TYPE_STRING:
-		break; /* valid */
-	case ITEM_TYPE_PATH:
-		if (   !value
-		    || !strlen (value)
-		    || !g_path_is_absolute (value)
-		    || !g_file_test (value, G_FILE_TEST_EXISTS)) {
-			g_set_error (info->error,
-			             NM_VPN_PLUGIN_ERROR,
-			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-			             "property '%s' file path '%s' is not absolute or does not exist",
-			             key, value);
-		}
-		break;
-	case ITEM_TYPE_INT:
-		errno = 0;
-		tmp = strtol (value, NULL, 10);
-		if (errno == 0 && tmp >= prop->int_min && tmp <= prop->int_max)
-			break; /* valid */
-
-		g_set_error (info->error,
-		             NM_VPN_PLUGIN_ERROR,
-		             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-		             "invalid integer property '%s' or out of range [%d -> %d]",
-		             key, prop->int_min, prop->int_max);
-		break;
-	case ITEM_TYPE_BOOLEAN:
-		if (!strcmp (value, "yes") || !strcmp (value, "no"))
-			break; /* valid */
-
-		g_set_error (info->error,
-		             NM_VPN_PLUGIN_ERROR,
-		             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-		             "invalid boolean property '%s' (not yes or no)",
-		             key);
-		break;
-	default:
-		g_set_error (info->error,
-		             NM_VPN_PLUGIN_ERROR,
-		             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-		             "unhandled property '%s' type %d",
-		             key, prop->type);
-		break;
 	}
 }
 
@@ -247,14 +223,14 @@ vpnc_watch_cb (GPid pid, gint status, gpointer user_data)
 	if (WIFEXITED (status)) {
 		error = WEXITSTATUS (status);
 		if (error != 0)
-			nm_warning ("vpnc exited with error code %d", error);
+			g_warning ("vpnc exited with error code %d", error);
 	}
 	else if (WIFSTOPPED (status))
-		nm_warning ("vpnc stopped unexpectedly with signal %d", WSTOPSIG (status));
+		g_warning ("vpnc stopped unexpectedly with signal %d", WSTOPSIG (status));
 	else if (WIFSIGNALED (status))
-		nm_warning ("vpnc died with signal %d", WTERMSIG (status));
+		g_warning ("vpnc died with signal %d", WTERMSIG (status));
 	else
-		nm_warning ("vpnc died from an unknown cause");
+		g_warning ("vpnc died from an unknown cause");
 
 	/* Reap child if needed. */
 	waitpid (priv->pid, NULL, WNOHANG);
@@ -314,12 +290,12 @@ nm_vpnc_start_vpnc_binary (NMVPNCPlugin *plugin, GError **error)
 							 G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, &stdin_fd,
 							 NULL, NULL, error)) {
 		g_ptr_array_free (vpnc_argv, TRUE);
-		nm_warning ("vpnc failed to start.  error: '%s'", (*error)->message);
+		g_warning ("vpnc failed to start.  error: '%s'", (*error)->message);
 		return -1;
 	}
 	g_ptr_array_free (vpnc_argv, TRUE);
 
-	nm_info ("vpnc started with pid %d", pid);
+	g_message ("vpnc started with pid %d", pid);
 
 	NM_VPNC_PLUGIN_GET_PRIVATE (plugin)->pid = pid;
 	vpnc_watch = g_child_watch_source_new (pid);
@@ -359,7 +335,7 @@ static void
 write_one_property (const char *key, const char *value, gpointer user_data)
 {
 	WriteConfigInfo *info = (WriteConfigInfo *) user_data;
-	guint32 type = ITEM_TYPE_UNKNOWN;
+	GType type = G_TYPE_INVALID;
 	int i;
 
 	if (info->error)
@@ -377,7 +353,7 @@ write_one_property (const char *key, const char *value, gpointer user_data)
 	}
 
 	/* Try the valid secrets table */
-	for (i = 0; type == ITEM_TYPE_UNKNOWN && valid_secrets[i].name; i++) {
+	for (i = 0; type == G_TYPE_INVALID && valid_secrets[i].name; i++) {
 		ValidProperty prop = valid_secrets[i];
 
 		if (!strcmp (prop.name, (char *) key)) {
@@ -387,14 +363,13 @@ write_one_property (const char *key, const char *value, gpointer user_data)
 		}
 	}
 
-	if (type == ITEM_TYPE_UNKNOWN) {
+	if (type == G_TYPE_INVALID) {
 		g_set_error (&info->error,
 		             NM_VPN_PLUGIN_ERROR,
 		             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
 		             "Config option '%s' invalid or unknown.",
 		             (const char *) key);
-		return;
-	}
+	}	
 
 	/* Don't write ignored secrets */
 	if (!strcmp (key, NM_VPNC_KEY_XAUTH_PASSWORD) && info->upw_ignored)
@@ -402,12 +377,12 @@ write_one_property (const char *key, const char *value, gpointer user_data)
 	if (!strcmp (key, NM_VPNC_KEY_SECRET) && info->gpw_ignored)
 		return;
 
-	if (type == ITEM_TYPE_STRING || type == ITEM_TYPE_PATH)
+	if (type == G_TYPE_STRING)
 		write_config_option (info->fd, "%s %s\n", (char *) key, (char *) value);
-	else if (type == ITEM_TYPE_BOOLEAN) {
+	else if (type == G_TYPE_BOOLEAN) {
 		if (!strcmp (value, "yes"))
 			write_config_option (info->fd, "%s\n", (char *) key);
-	} else if (type == ITEM_TYPE_INT) {
+	} else if (type == G_TYPE_INT) {
 		long int tmp_int;
 		char *tmp_str;
 
@@ -427,11 +402,11 @@ write_one_property (const char *key, const char *value, gpointer user_data)
 			             "Config option '%s' not an integer.",
 			             (const char *) key);
 		}
-	} else if (type == ITEM_TYPE_IGNORED) {
+	} else if (type == G_TYPE_NONE) {
 		/* ignored */
 	} else {
 		/* Just ignore unknown properties */
-		g_warning ("Don't know how to write property '%s' with type %d", key, type);
+		g_warning ("Don't know how to write property '%s' with type %s", key, g_type_name (type));
 	}
 }
 
@@ -446,6 +421,7 @@ nm_vpnc_config_write (gint vpnc_fd,
 	const char *default_username;
 	const char *pw_type;
 	const char *local_port;
+	NMSettingSecretFlags secret_flags = NM_SETTING_SECRET_FLAG_NONE;
 
 	default_username = nm_setting_vpn_get_user_name (s_vpn);
 
@@ -494,14 +470,25 @@ nm_vpnc_config_write (gint vpnc_fd,
 	info->fd = vpnc_fd;
 
 	/* Check for ignored user password */
-	pw_type = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD_TYPE);
-	if (pw_type && !strcmp (pw_type, NM_VPNC_PW_TYPE_UNUSED))
-		info->upw_ignored = TRUE;
+	if (nm_setting_get_secret_flags (NM_SETTING (s_vpn), NM_VPNC_KEY_XAUTH_PASSWORD, &secret_flags, NULL)) {
+		if (secret_flags & NM_SETTING_SECRET_FLAG_NOT_REQUIRED)
+			info->upw_ignored = TRUE;
+	} else {
+		pw_type = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD_TYPE);
+		if (pw_type && !strcmp (pw_type, NM_VPNC_PW_TYPE_UNUSED))
+			info->upw_ignored = TRUE;
+	}
 
 	/* Check for ignored group password */
-	pw_type = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_SECRET_TYPE);
-	if (pw_type && !strcmp (pw_type, NM_VPNC_PW_TYPE_UNUSED))
-		info->gpw_ignored = TRUE;
+	secret_flags = NM_SETTING_SECRET_FLAG_NONE;
+	if (nm_setting_get_secret_flags (NM_SETTING (s_vpn), NM_VPNC_KEY_SECRET, &secret_flags, NULL)) {
+		if (secret_flags & NM_SETTING_SECRET_FLAG_NOT_REQUIRED)
+			info->gpw_ignored = TRUE;
+	} else {
+		pw_type = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_SECRET_TYPE);
+		if (pw_type && !strcmp (pw_type, NM_VPNC_PW_TYPE_UNUSED))
+			info->gpw_ignored = TRUE;
+	}
 
 	nm_setting_vpn_foreach_data_item (s_vpn, write_one_property, info);
 	nm_setting_vpn_foreach_secret (s_vpn, write_one_property, info);
@@ -546,14 +533,39 @@ out:
 	return success;
 }
 
+static NMSettingSecretFlags
+get_pw_flags (NMSettingVPN *s_vpn, const char *secret_name, const char *type_name)
+{
+	const char *val;
+	NMSettingSecretFlags flags = NM_SETTING_SECRET_FLAG_NONE;
+
+	/* Try new flags value first */
+	if (nm_setting_get_secret_flags (NM_SETTING (s_vpn), secret_name, &flags, NULL))
+		return flags;
+
+	/* Otherwise try old "password type" value */
+	val = nm_setting_vpn_get_data_item (s_vpn, type_name);
+	if (val) {
+		if (g_strcmp0 (val, NM_VPNC_PW_TYPE_ASK) == 0)
+			return NM_SETTING_SECRET_FLAG_NOT_SAVED;
+		else if (g_strcmp0 (val, NM_VPNC_PW_TYPE_UNUSED) == 0)
+			return NM_SETTING_SECRET_FLAG_NOT_REQUIRED;
+
+		/* NM_VPNC_PW_TYPE_SAVE means FLAG_NONE */
+	}
+
+	return NM_SETTING_SECRET_FLAG_NONE;
+}
+
 static gboolean
 real_need_secrets (NMVPNPlugin *plugin,
                    NMConnection *connection,
-                   char **setting_name,
+                   char **out_setting_name,
                    GError **error)
 {
 	NMSettingVPN *s_vpn;
-	const char *pw_type;
+	NMSettingSecretFlags pw_flags;
+	const char *pw = NULL;
 
 	g_return_val_if_fail (NM_IS_VPN_PLUGIN (plugin), FALSE);
 	g_return_val_if_fail (NM_IS_CONNECTION (connection), FALSE);
@@ -568,20 +580,20 @@ real_need_secrets (NMVPNPlugin *plugin,
 		return FALSE;
 	}
 
-	pw_type = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_SECRET_TYPE);
-	if (!pw_type || strcmp (pw_type, NM_VPNC_PW_TYPE_UNUSED)) {
-		if (!nm_setting_vpn_get_secret (s_vpn, NM_VPNC_KEY_SECRET)) {
-			*setting_name = NM_SETTING_VPN_SETTING_NAME;
-			return TRUE;
-		}
+	/* User password */
+	pw = nm_setting_vpn_get_secret (s_vpn, NM_VPNC_KEY_SECRET);
+	pw_flags = get_pw_flags (s_vpn, NM_VPNC_KEY_SECRET, NM_VPNC_KEY_SECRET_TYPE);
+	if (!pw && !(pw_flags & NM_SETTING_SECRET_FLAG_NOT_REQUIRED)) {
+		*out_setting_name = NM_SETTING_VPN_SETTING_NAME;
+		return TRUE;
 	}
 
-	pw_type = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD_TYPE);
-	if (!pw_type || strcmp (pw_type, NM_VPNC_PW_TYPE_UNUSED)) {
-		if (!nm_setting_vpn_get_secret (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD)) {
-			*setting_name = NM_SETTING_VPN_SETTING_NAME;
-			return TRUE;
-		}
+	/* Group password */
+	pw = nm_setting_vpn_get_secret (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD);
+	pw_flags = get_pw_flags (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD, NM_VPNC_KEY_XAUTH_PASSWORD_TYPE);
+	if (!pw && !(pw_flags & NM_SETTING_SECRET_FLAG_NOT_REQUIRED)) {
+		*out_setting_name = NM_SETTING_VPN_SETTING_NAME;
+		return TRUE;
 	}
 
 	return FALSE;
@@ -610,7 +622,7 @@ real_disconnect (NMVPNPlugin   *plugin,
 		else
 			kill (priv->pid, SIGKILL);
 
-		nm_info ("Terminated vpnc daemon with PID %d.", priv->pid);
+		g_message ("Terminated vpnc daemon with PID %d.", priv->pid);
 		priv->pid = 0;
 	}
 
